/*////////////////////////////////////////////////////////////////////////////////

  Xilinx, Inc. 2006                 www.xilinx.com

  DCM Spread Spectrum Reference Design

//////////////////////////////////////////////////////////////////////////////

  File name :       dcm_clkgen_softspread.v

  Description :Creates triangular soft spread spectrum solution using DCM_CLKGEN
					with a state machine for controlling the programming port.
  
  Date - revision : Sep 1, 2009 - v 1.0

  Version 1.0: Jim T, 		09_01_09 Spartan-6 Release
  Version 0.9: Alireza K	07_21_06 


  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
              provided to you "as is". Xilinx and its licensors make and you
              receive no warranties or conditions, express, implied,
              statutory or otherwise, and Xilinx specifically disclaims any
              implied warranties of merchantability, non-infringement,or
              fitness for a particular purpose. Xilinx does not warrant that
              the functions contained in these designs will meet your
              requirements, or that the operation of these designs will be
              uninterrupted or error free, or that defects in the Designs
              will be corrected. Furthermore, Xilinx does not warrantor
              make any representations regarding use or the results of the
              use of the designs in terms of correctness, accuracy,
              reliability, or otherwise.

              LIMITATION OF LIABILITY. In no event will Xilinx or its
              licensors be liable for any loss of data, lost profits,cost
              or procurement of substitute goods or services, or for any
              special, incidental, consequential, or indirect damages
              arising from the use or operation of the designs or
              accompanying documentation, however caused and on any theory
              of liability. This limitation will apply even if Xilinx
              has been advised of the possibility of such damage. This
              limitation shall apply not-withstanding the failure of the
              essential purpose of any limited remedies herein.

  Copyright  2009 Xilinx, Inc.
  All rights reserved

	Modules: SStop  - State machine control DCM_CLKGEN programming ports
				SScontrol - Sub-block of SStop used to serialize programming 
				            data for DCM_CLKGEN
	Purpose: Controls DFS to hop between 2 frequencies to create spread 
	         spectrum clocking 

	USAGE:
	Use modfreq output to measure the modulation frequency
	
	The spread can be further adjusted using "FAST" and "SLOW" registers (7/5 
	is default for down spread)
	
	CLKFX_MULTIPLY_IN must be set to match DCM_CLKGEN to CLKFX_MULTIPLY to 
	ensure proper modulation.
	
	CLKFX_DIVIDE_IN must be set to match DCM_CLKGEN to CLKFX_DIVIDE to ensure 
	proper modulation. CLKFX_DIVIDE_IN must be at least 2 in order for sscontrol 
	to be able slow downt the modulation
	

//////////////////////////////////////////////////////////////////////////////*/


module DCM_CLKGEN_SOFTSPREAD  (
  	CLKIN,
	RST,

	CLKFX,
  	CLKFX180,
  	CLKFXDV,
	CLKIN_BUFG,
  	LOCKED,
  	PROGDONE,
  	STATUS,
  	FREEZEDCM,
  	PROGCLK,
  	PROGDATA,
 	PROGEN,
	MODFREQ
	);

  	input		CLKIN;
	input		RST;
  	input		FREEZEDCM;		//Must be connected to 1'b0 for correct operation
  	input		PROGCLK;			//Not being used. PROGEN being by SScontrol
  	input		PROGDATA;		//Not being used. PROGEN being by SScontrol
 	input		PROGEN;			//Not being used. PROGEN being by SScontrol

  	output	[2:1] STATUS;	//STATUS[2] defines if CLKFX has stopped
	output	CLKIN_BUFG;		//CLKIN => BUFG => "CLKIN_BUFG" output. BUFG is used within
									//DCM_CLKGEN_SOFTSPEAD. BUFG is brought out brought
									//out if needed for upper levels
	output	CLKFX;			//CLKFX => BUFG => "CLKFX" output
  	output	CLKFX180;		//CLKFX180 => BUFG => "CLKFX180" output
  	output	CLKFXDV;			//CLKFXDV => BUFG => "CLKFXDV" output
  	output	LOCKED;	  		//Connected to spread spectrum locked signal
  	output	PROGDONE; 		//Connected to CLKGEN PROGDONE
  	output	MODFREQ; 		//Use to measure modulation frequency		

	//SSTOP #(.CLKFX_DIVIDE_IN(2), .CLKFX_MULTIPLY_IN(7))	//Sets M/D parameters for sstop and sscontrol
	SSTOP #(.CLKFX_DIVIDE_IN(4), .CLKFX_MULTIPLY_IN(7)) 	//dcm_clkgen_softspread_m7d4 
										//CLKFX_DIVIDE_IN set to 4 to adjust for new DCM_CLKGEN settings
		SS ( 
			.SS_CLK(CLKFX), 					//Input from CLKFX (7/2) 3.5x
			.PROGCLK(CLKIN_BUFG), 			//Input from pixel clock
			.PROGEN(PROGEN_INT),				//Output connect to CLKGEN
			.PROGDATA(PROGDATA_INT),		//Output connect to CLKGEN
			.PROGDONE(PROGDONE), 			//Input from CLKGEN to indicate ready for next programming value
			.DFS_LOCKED( CLKGEN_LOCKED), 	//Input from DCM Lock
			.SSLOCK(LOCKED), 					//Additional Lock signal since DCM_CLKGEN is put into a mode that doesn't lock
			.SSMODE(2'b11),  			
			.MODFREQ(MODFREQ), 
			.RST(RST));  


	DCM_CLKGEN    #(
		.DFS_OSCILLATOR_MODE("PHASE_FREQ_LOCK"), 
		.CLKIN_PERIOD ("10"),
		//.SPREAD_SPECTRUM ("VIDEO_LINK_M2"),
		.SPREAD_SPECTRUM ("VIDEO_LINK_M1"), //dcm_clkgen_softspread_m7d4 Need to speed up modulation rate 
														//because CLKFX_DIVIDE has been increased
 		.CLKFX_MD_MAX("3.675"),
 		.CLKFX_MULTIPLY	(7),
		//.CLKFX_DIVIDE	(2) )	
 		.CLKFX_DIVIDE	(4) )	//dcm_clkgen_softspread_m7d4 Reducing CLKFX output to accomodate 
									//higher pixel clock frequencies
	dcm_clk (
		.CLKIN   	(CLKIN),
		.FREEZEDCM (1'b0),
		.PROGDATA		(PROGDATA_INT),
		.PROGEN 			(PROGEN_INT),
		.PROGCLK			(CLKIN_BUFG), 
		.PROGDONE		(PROGDONE),
		.RST     	(RST),
		.CLKFX   	(CLKFX_DCM),
		.CLKFX180	(CLKFX180_DCM),
		.CLKFXDV	(CLKFXDV_DCM),
		.LOCKED  	(CLKGEN_LOCKED),
		.STATUS  	(STATUS) ) ;


	BUFG 	clk_bufg	(.I(CLKIN),.O(CLKIN_BUFG) ) ;
	BUFG 	clkfx_bufg 	(.I(CLKFX_DCM),  .O(CLKFX) ) ;
	BUFG 	clkfxnot_bufg	(.I(CLKFX180_DCM), .O(CLKFX180) ) ;
	BUFG 	clkfxdv_bufg	(.I(CLKFXDV_DCM), .O(CLKFXDV) ) ;

endmodule