/*////////////////////////////////////////////////////////////////////////////////

  Xilinx, Inc. 2006                 www.xilinx.com

  DCM Spread Spectrum Reference Design

//////////////////////////////////////////////////////////////////////////////

  File name :       dcm_clkgen_softspread.v

  Description :Creates triangular soft spread spectrum solution using DCM_CLKGEN
					with a state machine for controlling the programming port.
  
  Date - revision : Sep 1, 2009 - v 1.0

  Version 1.0: Jim T, 		09_01_09
  Version 0.9: Alireza K	07_21_06 


  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
              provided to you "as is". Xilinx and its licensors make and you
              receive no warranties or conditions, express, implied,
              statutory or otherwise, and Xilinx specifically disclaims any
              implied warranties of merchantability, non-infringement,or
              fitness for a particular purpose. Xilinx does not warrant that
              the functions contained in these designs will meet your
              requirements, or that the operation of these designs will be
              uninterrupted or error free, or that defects in the Designs
              will be corrected. Furthermore, Xilinx does not warrantor
              make any representations regarding use or the results of the
              use of the designs in terms of correctness, accuracy,
              reliability, or otherwise.

              LIMITATION OF LIABILITY. In no event will Xilinx or its
              licensors be liable for any loss of data, lost profits,cost
              or procurement of substitute goods or services, or for any
              special, incidental, consequential, or indirect damages
              arising from the use or operation of the designs or
              accompanying documentation, however caused and on any theory
              of liability. This limitation will apply even if Xilinx
              has been advised of the possibility of such damage. This
              limitation shall apply not-withstanding the failure of the
              essential purpose of any limited remedies herein.

	Copyright  2006 Xilinx, Inc.
	All rights reserved

	Modules: SStop  - State machine control DCM_CLKGEN programming ports
				SScontrol - Sub-block of SStop used to serialize programming data for DCM_CLKGEN
	Purpose: Controls DFS to hop between 2 frequencies to create spread spectrum clocking 
	USAGE:
	SSMODE: 00: Freq align mode, 11: SS mode for measuring modulation frequency, 10: SS mode with toggling lock, 01: reserved
	MODFREQ OUTPUT is the modulation frequency
	Spread: Controlled by FAST/SLOW registers
	

//////////////////////////////////////////////////////////////////////////////*/


module SSTOP (SS_CLK, PROGCLK, PROGEN, PROGDATA, PROGDONE, DFS_LOCKED, RST, 
				SSMODE, MODFREQ, SSLOCK);
				output PROGEN, PROGDATA, MODFREQ, SSLOCK;
				input SS_CLK, PROGCLK, PROGDONE, DFS_LOCKED, RST;
				input [1:0] SSMODE;	// 00: Freq align mode without SS, 11: SS mode for measuring modulation frequency
											// 10: SS mode with toggling lock, 01: Freq align mode with SS 
				

				parameter OFFSET=8'h4;
				parameter FAST=8'h47;
				parameter SLOW=8'h45;
				parameter TOOFAST=FAST+OFFSET;
				parameter TOOSLOW=SLOW-OFFSET;

				parameter integer CLKFX_DIVIDE_IN = 2; 
				parameter integer CLKFX_MULTIPLY_IN = 7; 


				reg [9:0] CNTR;
				reg [7:0] LASTFREQDEV;
				reg [2:0] WAITCNTR;
				reg COMPREAD;
				reg [1:0] FREQSEL;
				reg GO2NEXT, PREV_GO2NEXT;
				reg [6:0] GOCNTR;
				
				reg COMPVALID_FX, COMPVALID_META, COMPREAD_FX, COMPREAD_META;
				reg [9:0] FXCOUNT;
				reg [7:0] REFCOUNT;
				wire RSTCOMP, COMPVALID, RSTCOMP_FX;
			assign COMPVALID = REFCOUNT[5] ;
			always @ (posedge SS_CLK or posedge RST) 
					  if (RST) begin  COMPVALID_META <=0; COMPVALID_FX<=0; 
						COMPREAD_META <=0; COMPREAD_FX<=0; end 
					  else begin 
						COMPVALID_META <= COMPVALID;
						COMPVALID_FX <= COMPVALID_META;
						COMPREAD_META <= COMPREAD;
						COMPREAD_FX <= COMPREAD_META;        
					  end
			always @ (posedge SS_CLK or posedge RSTCOMP_FX ) 
					  if (RSTCOMP_FX) begin FXCOUNT <= 0; end
					  else if ( !COMPVALID_FX ) FXCOUNT <= FXCOUNT + 1;
					  else FXCOUNT <= FXCOUNT;
								  
			always @ (posedge PROGCLK or posedge RSTCOMP )
					  if (RSTCOMP) begin REFCOUNT <= 0; end
					  else if ( !COMPVALID ) REFCOUNT <= REFCOUNT + 1;
				else REFCOUNT <= REFCOUNT;
			wire [9:0] Freqdev = FXCOUNT - ((10'h20 * CLKFX_MULTIPLY_IN / CLKFX_DIVIDE_IN ) - 10'h48);//jt test
			

			assign RSTCOMP = COMPREAD | RST;
			assign RSTCOMP_FX = COMPREAD_FX | RSTCOMP;
			reg MODFREQ;
			wire RDY4NEXT; 
			wire SPEEDUP = (LASTFREQDEV<SLOW);
			wire SLOWDOWN = (LASTFREQDEV>FAST);
			wire IS2FAST = (LASTFREQDEV>TOOFAST);
			wire IS2SLOW = (LASTFREQDEV<TOOSLOW);
			reg IS2SLOW_REG, IS2FAST_REG, SLOWDOWN_REG, SPEEDUP_REG;
			reg LOCK1st, B4LOCK, SSLOCK;		
			reg [4:0] LOCKCNTR; 				 
			reg W4SPEEDUP, W4SLOWDOWN, W4LOCK;	 
				

			always @ (posedge PROGCLK or posedge RST )
					 if (RST) begin 
						CNTR <= 0; COMPREAD <= 0; WAITCNTR <= 0; LASTFREQDEV <= 8'h48; 
						LOCK1st <= 0; W4SPEEDUP<=1'b0; W4SLOWDOWN<=1'b0; SSLOCK <=1'b0;  
						LOCKCNTR <=0; W4LOCK<=1'b1;   
						MODFREQ <= 0;
						end
					  else begin
						if (COMPVALID || COMPREAD)  WAITCNTR <= WAITCNTR + 1;
						else WAITCNTR <= 0;        	

						if ( COMPVALID && WAITCNTR[1] ) LASTFREQDEV <= Freqdev[7:0]; 
						else if (WAITCNTR[2]) COMPREAD <= 1'b1;
						else begin COMPREAD <= 0; LASTFREQDEV <= LASTFREQDEV; end
						
						if ( (SLOWDOWN || SPEEDUP) && !LOCK1st ) CNTR <= CNTR+1;
						else CNTR <= 0;


					if (SSMODE == 2'b00) FREQSEL <= 2'b11;
					else if ( ((IS2FAST_REG || IS2SLOW_REG || GOCNTR[5]) && (SSMODE==2'b10)) || (SSMODE==2'b01) ) 
					
								  FREQSEL <= 2'b00;
					else if (RDY4NEXT || SPEEDUP_REG ) FREQSEL <= 2'b10; 
								 else if ( SLOWDOWN_REG ) FREQSEL <= 2'b01; 
								 else FREQSEL <= FREQSEL;

					if (SPEEDUP_REG ) MODFREQ <= 1'b1;
					else if (SLOWDOWN_REG ) MODFREQ <= 1'b0;
					else MODFREQ <= MODFREQ;
					
					PREV_GO2NEXT <= GO2NEXT;
					if (!SSMODE[1]) GO2NEXT <= LOCK1st & !B4LOCK;
					else GO2NEXT <= ( (!W4LOCK&(SPEEDUP_REG | SLOWDOWN_REG)) | RDY4NEXT);	

					IS2FAST_REG <= IS2FAST; IS2SLOW_REG <= IS2SLOW;
					SPEEDUP_REG <= SPEEDUP & !W4SLOWDOWN; 
					SLOWDOWN_REG <= SLOWDOWN & !W4SPEEDUP;

			// This portion compensates for inconsistencies of fabric freq measurement
			// prevents double hops in the same direction
			// useful for things like modulation frequency

					if (SPEEDUP && CNTR[4] && !CNTR[9]) W4SLOWDOWN <= 1'b1;
					else if ( (SLOWDOWN && CNTR[4]) || (SPEEDUP && CNTR[9]) || IS2SLOW_REG || (FREQSEL[0]==FREQSEL[1]) )
						 W4SLOWDOWN <=1'b0;
					else W4SLOWDOWN <= W4SLOWDOWN;		
					if (SLOWDOWN && CNTR[4] && !CNTR[9]) W4SPEEDUP <= 1'b1;
					else if ( (SPEEDUP && CNTR[4]) || (SLOWDOWN && CNTR[9]) || IS2FAST_REG || (FREQSEL[0]==FREQSEL[1]) )
						 W4SPEEDUP <=1'b0;
					else W4SPEEDUP <= W4SPEEDUP;
					 
			 // This portion is added to create an SS lock
					if (DFS_LOCKED && B4LOCK == 0)  LOCK1st <= 1'b1;
					else B4LOCK <= LOCK1st;
					if ( !SSMODE[1] ) SSLOCK <= DFS_LOCKED & B4LOCK & LOCK1st;
					else begin
						if (LOCK1st && !LOCKCNTR[4] && !PREV_GO2NEXT && GO2NEXT) LOCKCNTR <= LOCKCNTR+1;
						if (LOCKCNTR[4] && SSMODE[0]) SSLOCK <= !(IS2FAST_REG | IS2SLOW_REG);
						if (LOCKCNTR[4] && !SSMODE[0]) SSLOCK <= !(IS2FAST_REG | IS2SLOW_REG | GOCNTR[6]);
					end
								
			 // This portion is for occational DFS lock of SSMODE 2'b10
					if ( RDY4NEXT || !LOCK1st ) GOCNTR <=0;
					else if( !PREV_GO2NEXT && GO2NEXT ) GOCNTR <= GOCNTR + 1;
					if ( GOCNTR[5] && (FREQSEL==2'b00) && PREV_GO2NEXT && !GO2NEXT && (SSMODE==2'b10) ) 
							W4LOCK<=1'b1;
					else if (RDY4NEXT) W4LOCK<=1'b0;
					else W4LOCK <= W4LOCK;
			 
					  end         

				
				sscontrol #(.CLKFX_DIVIDE_IN(CLKFX_DIVIDE_IN), .CLKFX_MULTIPLY_IN(CLKFX_MULTIPLY_IN))
					SS (.CLK(PROGCLK), .HOPCTL(PROGEN), .HOPCMD(PROGDATA),
					 .HOPRDY(PROGDONE), .DFS_LOCKED(DFS_LOCKED), .NEXTFREQ(GO2NEXT), 
					 .FREQSEL(FREQSEL), .READYFORNEXT(RDY4NEXT), .GRESET(RST));

endmodule
