/*////////////////////////////////////////////////////////////////////////////////

  Xilinx, Inc. 2006                 www.xilinx.com

  DCM Spread Spectrum Reference Design

//////////////////////////////////////////////////////////////////////////////

  File name :       ssclkgen.v

  Description :     Example usgage of Spread Spectrum clock generation reference design
  Date - revision : Sep 1, 2009 - v 1.0

  Version 1.0: Jim T, 09_15_09, Spartan-6 Release
  Version 0.9: Jim T, 02_15_08, packaged with TX core of XAPP486  
  Version 0.8: Alireza K, 07_21_06, Initial Release 
 

  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
              provided to you "as is". Xilinx and its licensors make and you
              receive no warranties or conditions, express, implied,
              statutory or otherwise, and Xilinx specifically disclaims any
              implied warranties of merchantability, non-infringement,or
              fitness for a particular purpose. Xilinx does not warrant that
              the functions contained in these designs will meet your
              requirements, or that the operation of these designs will be
              uninterrupted or error free, or that defects in the Designs
              will be corrected. Furthermore, Xilinx does not warrantor
              make any representations regarding use or the results of the
              use of the designs in terms of correctness, accuracy,
              reliability, or otherwise.

              LIMITATION OF LIABILITY. In no event will Xilinx or its
              licensors be liable for any loss of data, lost profits,cost
              or procurement of substitute goods or services, or for any
              special, incidental, consequential, or indirect damages
              arising from the use or operation of the designs or
              accompanying documentation, however caused and on any theory
              of liability. This limitation will apply even if Xilinx
              has been advised of the possibility of such damage. This
              limitation shall apply not-withstanding the failure of the
              essential purpose of any limited remedies herein.

	Copyright  2006 Xilinx, Inc.
	All rights reserved

	Modules: DCM_CLKGEN_SOFTSPREAD
	Purpose: Example showing how to use XAPP_??? reference design
	USAGE:
		Connect Differential clock to "CLKIN_P", "CLKIN_N"
		Connect active low reset signal to "RST"
		
		outputs for measuring are provided with:
		CLKREF_P, CLKREF_N - differential output tied to input clock to use as a baseline
		CLKPLL_P, CLKPLL_N - differential output driven by PLL and OSERDES2
		

//////////////////////////////////////////////////////////////////////////////*/


module XAPP_SS ( 
	CLKIN_P, CLKIN_N, 		//Differential clock input
	RST, 							//Active low system reset signal

	CLKREF_P, CLKREF_N, 		//Differential ouptut used to measure the frequency of clock input
	CLKPLL_P, 	CLKPLL_N,	//Differential output used to measure PLL output frequency

	LOCKED_PLL,					//PLL lock signal
	DCM_LCKD, 					//DCM_CLKGEN lock signal
	CLK_LCKD_OUT, 				//DCM_CLKGEN lock signal (same as dcm
	SSMOD_FREQ);				//Single ended signal used for measuring Modulation Frequency

	parameter integer SERIALIZATION = 7;	//Defines serialiation rate for OSERDES2

	input		RST;
	wire		RST_ACTIVELOW = ! RST;
	input 	CLKIN_P, CLKIN_N;

	output	CLKREF_P, CLKREF_N;
	output 	CLKPLL_P, CLKPLL_N;
	output 	LOCKED_PLL;
	output 	CLK_LCKD_OUT, DCM_LCKD, SSMOD_FREQ;

	wire 		CLKX3P5DCM;
	wire 		CLKX3P5NOTDCM;
	wire 		CLKX3P5 ;
	wire 		CLKX3P5NOT ;
	wire 		CLKDCM ;
	wire 		CLKIN; 
	wire 		CLK ;
	wire 		[6:0]	SHIFTERP ;
	wire 		[6:0]	SHIFTERP_D ;
	wire 		PSDONE, PSINCDEC, PSEN;
	wire 		CLK_LCKD;
	reg  		DCM_LCKD;

	wire 		CS_RST;
	wire 		DIVLOCKED;

	IBUFGDS	CLK_ibuf 	(.I(CLKIN_P),	.IB(CLKIN_N), .O(CLKIN_IBUFG) );	
	BUFIO2	#(.DIVIDE_BYPASS("TRUE"))CLK_bufio2 	(.I(CLKIN_IBUFG),	.DIVCLK(CLKIN), .IOCLK(), .SERDESSTROBE());	

	DCM_CLKGEN_SOFTSPREAD i_dcm_clkgen_softspread(
		.CLKIN   	(CLKIN),
		.FREEZEDCM 	(), 		//Unused. Controlled by DCM_CLKGEN_SOFTSPREAD
		.PROGDATA	(), 		//Unused. Controlled by DCM_CLKGEN_SOFTSPREAD
		.PROGEN 		(),		//UnusedC. Controlled by DCM_CLKGEN_SOFTSPREAD
		.PROGCLK		(), 		//Unused. Controlled by DCM_CLKGEN_SOFTSPREAD
		.PROGDONE	(), 
		.RST     	(RST_ACTIVELOW),
		.CLKFX   	(CLKX3P5),
		.CLKFX180	(CLKX3P5NOT),
		.LOCKED  	(CLK_LCKD_OUT),
		.STATUS  	(),
		.CLKIN_BUFG(CLK),					   
		.MODFREQ(SSMOD_FREQ) ) ;

	always DCM_LCKD <= CLK_LCKD_OUT;
	assign RST_PLL = RST_ACTIVELOW &  CLK_LCKD_OUT;

	// PLL required to directly drive BUFPLL for OSERDES as used in video designs //
	PLL_BASE #(
			.BANDWIDTH 				("high"),
			.CLKIN_PERIOD			( 2.857),
			.CLKOUT0_DIVIDE		(1),
			.CLKOUT0_PHASE 		( 0.000),
			.CLKOUT0_DUTY_CYCLE	(0.500),
			.CLKOUT1_DIVIDE		(SERIALIZATION),
			.CLKOUT1_PHASE 		(0.000),
			.CLKOUT1_DUTY_CYCLE	(0.500),
			.DIVCLK_DIVIDE 		(1),
			.CLKFBOUT_MULT 		(2),
			.CLKFBOUT_PHASE 		(0.0),
			.CLK_FEEDBACK			("CLKFBOUT"),
			.REF_JITTER 			(0.030000))
		PLL_0 ( 
			.CLKFBOUT(CLKFB), .CLKFBIN(CLKFB), 
			.CLKIN(CLKX3P5), 
			.CLKOUT0(CLKOUT0),
			.CLKOUT1(CLKOUT1),
			.LOCKED(LOCKED_PLL), 
			.RST(RST_PLL)
		);

	BUFG inst_pll (.I(CLKOUT0), .O(CLKOUT0_bufg));
	BUFG inst_pll1 (.I(CLKOUT1), .O(CLKOUT1_BUFG));

	//Creating reference clock signal to measure the input clock frequency
	ODDR2 	#(.DDR_ALIGNMENT("NONE")) oddr_clkref (.C0(CLK), .C1(! CLK), .D0(1'b0), .D1(1'b1), .CE(), .R(), .S(), .Q(clkref_bufg)) ;
	OBUFDS	lvds_clkref_obuf	(.I(clkref_bufg), .O(CLKREF_P),  .OB(CLKREF_N) ); 


	BUFPLL #(
      .DIVIDE			(SERIALIZATION))         // PLLIN0 divide-by value to produce SERDESSTROBE (1 to 8); default 1
	rx_bufpll_inst2 (
      .PLLIN			(CLKOUT0),        		// PLL Clock input
      .GCLK				(CLKOUT1_BUFG), 			// Global Clock input
      .LOCKED			(LOCKED_PLL),           // Clock0 locked input
      .IOCLK			(IOCLK1), 					// Output PLL Clock
      .LOCK				(LOCK_PLL_OUT),			// BUFPLL Clock and strobe locked
      .SERDESSTROBE	(SERDESSTROBE1)) ; 		// Output SERDES strobe

	//Output logic stage set to recreate a 4:3 duty cycle LVDS clock to measure against input clock.
	//Using "SERIALIZATION" the design can be scaled to support up to 8:1 serialization 
	OSERDES2 #(
		.DATA_WIDTH     	(SERIALIZATION), 		// SERDES word width.  This should match the setting is BUFPLL
		.DATA_RATE_OQ     ("SDR"), 				// <SDR>, DDR
		.DATA_RATE_OT     ("SDR"), 				// <SDR>, DDR
		.SERDES_MODE    	("MASTER"), 			// <DEFAULT>, MASTER, SLAVE
		.OUTPUT_MODE 		("DIFFERENTIAL"))
	oserdes_m (
		.OQ       		(CLKREGPLL),
		.OCE     		(1'b1),
		.CLK0    		(IOCLK1),
		.CLK1    		(1'b0),
		.IOCE    		(SERDESSTROBE1),
		.RST     		(1'b0),
		.CLKDIV  		(CLKOUT1_BUFG),
		.D4  				(1'b1),
		.D3  				(1'b1),
		.D2  				(1'b1),
		.D1  				(1'b1),
		.TQ  				(),
		.T1 				(1'b0),
		.T2 				(1'b0),
		.T3 				(1'b0),
		.T4 				(1'b0),
		.TRAIN    		(1'b0),
		.TCE	   		(1'b0),
		.SHIFTIN1 		(),			
		.SHIFTIN2 		(),			
		.SHIFTIN3 		(SMC),			// Cascade output D data from slave
		.SHIFTIN4 		(SMD),			// Cascade output T data from slave
		.SHIFTOUT1 		(MSA),			// Cascade input D data to slave
		.SHIFTOUT2 		(MSB),			// Cascade input T data to slave
		.SHIFTOUT3 		(),			
		.SHIFTOUT4 		()) ;		

	OSERDES2 #(
		.DATA_WIDTH    (SERIALIZATION), 	
		.DATA_RATE_OQ  ("SDR"), 		
		.DATA_RATE_OT  ("SDR"), 		
		.SERDES_MODE   ("SLAVE"), 		
		.OUTPUT_MODE 	("DIFFERENTIAL"))
	oserdes_s (
		.OQ       		(),
		.OCE     		(1'b1),
		.CLK0    		(IOCLK1),
		.CLK1    		(1'b0),
		.IOCE    		(SERDESSTROBE1),
		.RST     		(1'b0),
		.CLKDIV  		(CLKOUT1_BUFG),
		.D4  				(1'b1),
		.D3  				(1'b0),
		.D2  				(1'b0),
		.D1  				(1'b0),
		.TQ  				(),
		.T1 				(1'b0),
		.T2 				(1'b0),
		.T3  				(1'b0),
		.T4  				(1'b0),
		.TRAIN 			(1'b0),
		.TCE	 			(1'b0),
		.SHIFTIN1 		(MSA),		// Cascade input D from Master
		.SHIFTIN2 		(MSB),		// Cascade input T from Master
		.SHIFTIN3 		(),			
		.SHIFTIN4 		(),			
		.SHIFTOUT1 		(),			
		.SHIFTOUT2 		(),			
		.SHIFTOUT3 		(SMC),   	// Cascade output D data to Master
		.SHIFTOUT4 		(SMD)) ; 	// Cascade output T data to Master

	OBUFDS	lvds_pll_obuf	(.I(CLKREGPLL), .O(CLKPLL_P),  .OB(CLKPLL_N) ); //Creating duplicate as a baseline matching typical dcm usage

	
endmodule
